package flink.zanxiangnet.ad.monitoring.maxcompute;

import com.aliyun.odps.data.Record;
import com.aliyun.odps.data.RecordWriter;
import com.aliyun.odps.tunnel.TableTunnel;
import flink.zanxiangnet.ad.monitoring.util.JsonUtil;
import flink.zanxiangnet.ad.monitoring.util.bean.BeanUtil;
import flink.zanxiangnet.ad.monitoring.util.bean.FieldInfo;
import flink.zanxiangnet.ad.monitoring.util.bean.ObjectUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

/**
 * 实际执行 MaxCompute写入逻辑
 */
public class MaxComputeWriter<T> implements Runnable {
    private static final Logger log = LoggerFactory.getLogger(MaxComputeWriter.class);

    private final TableTunnel.UploadSession uploadSession;
    private final RecordWriter recordWriter;
    private final Class<T> clazz;
    private final List<T> dataList;

    public MaxComputeWriter(Class<T> clazz, TableTunnel.UploadSession uploadSession, RecordWriter recordWriter, List<T> dataList) {
        this.clazz = clazz;
        this.uploadSession = uploadSession;
        this.recordWriter = recordWriter;
        this.dataList = dataList;
    }

    @Override
    public void run() {
        if (CollectionUtils.isEmpty(dataList)) {
            return;
        }
        List<FieldInfo> fieldInfoList = BeanUtil.parseBeanField(clazz);
        try {
            int i = 0;
            for(T t : dataList) {
                Record record = uploadSession.newRecord();
                for (FieldInfo fieldInfo : fieldInfoList) {
                    Object obj = fieldInfo.getGetMethod().invoke(t);
                    switch (fieldInfo.getColumnType()) {
                        case TINYINT:
                        case SMALLINT:
                        case INT:
                        case BIGINT:
                            record.setBigint(fieldInfo.getColumnName(), ObjectUtil.toLong(obj));
                            break;
                        case FLOAT:
                        case DOUBLE:
                            record.setDouble(fieldInfo.getColumnName(), ObjectUtil.toDouble(obj));
                            break;
                        case DECIMAL:
                            record.setDecimal(fieldInfo.getColumnName(), ObjectUtil.toDecimal(obj));
                            break;
                        case VARCHAR:
                        case CHAR:
                        case STRING:
                            record.setString(fieldInfo.getColumnName(), obj == null ? null : obj.toString());
                            break;
                        case DATE:
                        case TIMESTAMP:
                        case DATETIME:
                            record.setDatetime(fieldInfo.getColumnName(), ObjectUtil.toDate(obj));
                            break;
                        case BOOLEAN:
                            record.setBoolean(fieldInfo.getColumnName(), ObjectUtil.toBoolean(obj));
                            break;
                        default:
                            throw new RuntimeException("Unknown column type: " + JsonUtil.toString(fieldInfo));
                    }
                }
                if (i++ < 20) {
                    System.out.println(JsonUtil.toString(record));
                }
                recordWriter.write(record);
            }
        } catch (IOException | InvocationTargetException | IllegalAccessException e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("Failed writer to MaxCompute!!! Error msg: " + e.getMessage());
        }
    }
}