package flink.zanxiangnet.ad.monitoring.maxcompute.sink;

import com.aliyun.odps.Odps;
import com.aliyun.odps.account.Account;
import com.aliyun.odps.account.AliyunAccount;
import com.aliyun.odps.tunnel.TableTunnel;
import flink.zanxiangnet.ad.monitoring.maxcompute.MaxComputeLog;
import flink.zanxiangnet.ad.monitoring.util.JsonUtil;
import flink.zanxiangnet.ad.monitoring.util.bean.BeanUtil;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Map;

public class MaxComputeSink<IN> extends RichSinkFunction<IN> {
    private static final Logger log = LoggerFactory.getLogger(MaxComputeSink.class);

    // 对象锁，防止MaxCompute的 Tunnel对象多次初始化
    private static final Object DUMMY_LOCK = new Object();

    private final Class<IN> clazz;
    // private final Method partitionFieldMethod;
    private final String methodName;

    private volatile transient MaxComputeSinkBuffer<IN> sinkBuffer;

    public MaxComputeSink(Class<IN> clazz, String partitionFieldName) {
        this.clazz = clazz;
        methodName = "get" + partitionFieldName.substring(0, 1).toUpperCase() + partitionFieldName.substring(1);
        /*try {
            this.clazz = clazz;
            Field field = clazz.getDeclaredField(partitionFieldName);
            partitionFieldMethod = BeanUtil.methodOfGet(clazz, field);
        } catch (NoSuchFieldException e) {
            throw new RuntimeException("Can't found field: " + partitionFieldName);
        } catch (NoSuchMethodException e) {
            throw new RuntimeException("Can't found getMethod: " + partitionFieldName);
        }*/
    }

    @Override
    public void open(Configuration config) {
        if (sinkBuffer == null) {
            synchronized (DUMMY_LOCK) {
                if (sinkBuffer == null) {
                    Map<String, String> params = getRuntimeContext()
                            .getExecutionConfig()
                            .getGlobalJobParameters()
                            .toMap();

                    Account account = new AliyunAccount("LTAI5tFuLw65UsH3tqru2K1h", "p1F8my4ovgcEfs3HVORdmeLlLUUKRp");
                    Odps odps = new Odps(account);
                    odps.getRestClient().setRetryLogger(new MaxComputeLog());
                    odps.setEndpoint("http://service.cn-hangzhou.maxcompute.aliyun.com/api");
                    odps.setDefaultProject("ZxData_dev");
                    TableTunnel tunnel = new TableTunnel(odps);
                    tunnel.setEndpoint("http://dt.cn-hangzhou.maxcompute.aliyun.com");
                    sinkBuffer = new MaxComputeSinkBuffer<>(360000L, 64000L, 6, clazz, tunnel, "ZxData_dev", "ad_data_of_minute_ods");
                }
            }
        }
    }

    /**
     * 将值写入到 Sink。每个值都会调用此函数
     * @param value
     * @param context
     * @throws Exception
     */
    @Override
    public void invoke(IN value, Context context) throws Exception {
        Object obj = clazz.getMethod(methodName).invoke(value);
        if (obj == null) {
            throw new RuntimeException("The value of the partition field cannot be null!!!");
        }
        sinkBuffer.put(obj.toString(), value);
    }

    @Override
    public void close() throws Exception {
        if (sinkBuffer != null) {
            synchronized (DUMMY_LOCK) {
                sinkBuffer.close();
            }
        }

        super.close();
    }
}
